; Copyright (c) 2021, rrc-open-firmware FM10K-Documentation Contributors
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions are met:
;
;     * Redistributions of source code must retain the above copyright notice,
;       this list of conditions and the following disclaimer.
;     * Redistributions in binary form must reproduce the above copyright
;       notice, this list of conditions and the following disclaimer in the
;       documentation and/or other materials provided with the distribution.
;     * Neither the name of the copyright holder nor the names of its contributors
;       may be used to endorse or promote products derived from this software
;       without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
; DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
; CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
; OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


init_startClocks:
    MOV PLL_PCIE_CTRL,      0xfc640a        ; Nreset = 0, Enable = 1, Halt = 0, RefDiv = 1, FbDiv4 = 0, FbDiv255 = 0x19, OutDiv = 63, OutMuxSel = 0 (PCIE_REFCLK)
    MOV PLL_EPL_CTRL,       0xfcb81a        ; Nreset = 0, Enable = 1, Halt = 0, RefDiv = 3, FbDiv4 = 0, FbDiv255 = 0x2E, OutDiv = 63, OutMuxSel = 0
    MOV PLL_FABRIC_CTRL,    0x1cbc1a        ; Nreset = 0, Enable = 1, Halt = 0, RefDiv = 3, FbDiv4 = 0, FbDiv255 = 0x2F, OutDiv = 7, OutMuxSel = 0

    SET PLL_PCIE_STAT,      0x40,   0x40    ; Value & Mask, set MiscCtrl, "Fast Calibration Mode" = 0, "Asynchronous load signal for PLL output divider." = 1
    SET PLL_PCIE_STAT,      0,      0x40    ; Value & Mask, set MiscCtrl all 0 (?)
    SET PLL_EPL_STAT,       0x40,   0x40    ; Value & Mask, set MiscCtrl, "Fast Calibration Mode" = 0, "Asynchronous load signal for PLL output divider." = 1
    SET PLL_EPL_STAT,       0,      0x40    ; Value & Mask, set MiscCtrl all 0 (?)

    BEQ CHIP_VERSION,       platform_STEPPING_A0, 0x7f, @.skip_XPLL_init ; Lower stepping did not have some initialization steps
        MOV PCIE_XPLL_CTRL,     0x250c81    ; RefDiv = 1, FbDiv4 = 0, FbDiv255 = 0x19, OutDiv = 0xA, MiscCtrl[0] = 1 "Fast Calibration Mode"
        MOV PCIE_CLK_CTRL,      0xf0        ; Nreset[0-3] = 0, Enable[0-3] = 1, Halt[0-3] = 0
    .skip_XPLL_init:
    WAIT                    50


    MOV PLL_PCIE_CTRL,      0xfc640b        ; same as initial config
    MOV PLL_EPL_CTRL,       0xfcb81b        ; same as initial config
    MOV PLL_FABRIC_CTRL,    0x1cbc1b        ; same as initial config

    BEQ CHIP_VERSION,       platform_STEPPING_A0, 0x7f, @.skip_CLK_set
        MOV PCIE_CLK_CTRL,      0xff        ; Bring it back out of halt? Nreset[0-3] = 0, Enable[0-3] = 1, Halt[0-3] = 1
    .skip_CLK_set:
    WAIT                    100000

    MOV PLL_PCIE_CTRL,      0x01fc640b      ; Bring it back to normal operation. Nreset = 1, Enable = 1, Halt = 0, RefDiv = 1, FbDiv4 = 0, FbDiv255 = 0x19, OutDiv = 63, OutMuxSel = 1 (PLL_PCIE_CLK)
    MOV PLL_EPL_CTRL,       0x01fcb81b      ; Bring it back to normal operation. Nreset = 1, Enable = 1, Halt = 0, RefDiv = 3, FbDiv4 = 0, FbDiv255 = 0x2E, OutDiv = 63, OutMuxSel = 1 (PLL_PCIE_CLK)
    MOV PLL_FABRIC_CTRL,    0x011cbc1b      ; Bring it back to normal operation. Nreset = 1, Enable = 1, Halt = 0, RefDiv = 3, FbDiv4 = 0, FbDiv255 = 0x2F, OutDiv = 7, OutMuxSel = 1 (PLL_PCIE_CLK)

    WAIT                    10
    RET


; FASTCALL void (uint OutDiv)
init_PCIE_setClockOutDiv:
    SHL R0, P0, 18 ; P0 << 18
    AND R0, R0, 0x00fc0000

    MOV R1, PLL_PCIE_CTRL
    AND R1, R1, 0xff03ffff

    OR R1, R1, R0
    ; PLL_PCIE_CTRL = (PLL_PCIE_CTRL & 0xff03ffff) | ((P0 << 18) & 0x00fc0000);


    MOV PLL_PCIE_CTRL, R1 ; Set OutDiv
    SET PLL_PCIE_STAT, 0x40, 0x40 ; Value & Mask, set MiscCtrl, "Fast Calibration Mode" = 0, "Asynchronous load signal for PLL output divider." = 1
    SET PLL_PCIE_STAT, 0x00, 0x40 ; Value & Mask, set MiscCtrl all 0 (?)
    RET

; FASTCALL void(bool useSystimeClockSource)
init_setClockSource:
    BEQ P0,                 0, 0x1, @.return        ; If clock source is defined as system = 0
        SET DEVICE_CFG,     0x10000, 0x10000  ; set DEVICE_CFG.SystimeClockSource to IEEE1588_REFCLK (default PCIE_REFCLK)
    .return:
    RET

init_markClockStable:
    SET SOFT_RESET, 0, 0x1      ; Clear ColdReset to mark clocks are set and stable
    MOV api_BSM_STATUS, 0x0a    ; Set status for switch manager to know
    MOV api_DE_COLD_RESET_STATUS, api_BSM_STATUS
    RET